#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <GL/glut.h>

#define RAYON 0.8
#define PI 3.14
#define NB_FAISCEAUX 720

GLfloat ds[NB_FAISCEAUX], dt[NB_FAISCEAUX];

GLfloat	diffuseColor[] = {0.61424,0.64136,0.64136};
GLfloat	ambientColor[] = {0.1745,0.01175,0.01175};
GLfloat	redColor[] = {1.,0.,0.};
GLfloat	specularColor[] = {0.727811,0.626959,0.626959};
GLfloat lumierePosition[] = {0.5, 0.5, 3.0, 0.0};
GLfloat lumierePosition2[] = {0., 0., 0., 0.};

static void render(void) {
  static GLfloat x, y, z, sigma = PI/4., tetha = 0., yRot = 0.0;
  int i, maxEtoiles;
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glPushMatrix();
  glLoadIdentity();
  glTranslatef(0.0,0.0,-5.0);
	
  glEnable(GL_LIGHTING);
  glColor4f(1.0,1.0,1.0,1.0);
  glBegin(GL_LINES);
  maxEtoiles = (int)(2500.*fabs(sin(yRot/25.)));
  for(i=0;i<maxEtoiles;i++) {
    x = (rand()%6000-3000)/1000.;
    y = (rand()%6000-3000)/1000.;
    z = (rand()%4000-2000)/1000.;
    glVertex3f(x, y, z);
    glVertex3f(x, y, z+0.25);
  }
  glEnd();

  glRotatef(yRot,0.0,1.0,0.0);
  glDisable(GL_LIGHTING);

  glBegin(GL_TRIANGLES);
  for(i=0;i<NB_FAISCEAUX;i++) {
    glColor4f(1.0,0.0,0.0,0.2);
    glVertex3f(0.0,0.0,0.0);
    x = RAYON*cos(sigma+ds[i])*cos(tetha+dt[i]);
    y = RAYON*sin(sigma+ds[i]);
    z = RAYON*cos(sigma+ds[i])*sin(tetha+dt[i]);
    glColor4f(1.0,1.0,0.0,0.3);
    glVertex3f(x, y, z);
    
    sigma -= 0.02;
    
    x = RAYON*cos(sigma+ds[i])*cos(tetha+dt[i]);
    y = RAYON*sin(sigma+ds[i]);
    z = RAYON*cos(sigma+ds[i])*sin(tetha+dt[i]);
    glVertex3f(x, y, z);
    
    sigma += 0.02;
  }
  glEnd();
  tetha += 0.002;
  sigma += 0.005;
	
  glEnable(GL_LIGHTING);

  glEnable(GL_LIGHT1);
  glDisable(GL_BLEND);
  glColor4f(1.,0.,0.,1.);
  glutSolidSphere(0.45,30,20);
  glEnable(GL_BLEND);
  glDisable(GL_LIGHT1);

  glDisable(GL_LIGHTING);
  glColor4f(1.0,0.0,0.0,0.1);
  glutSolidSphere(0.53,12,10);
  glutSolidSphere(0.54,10,12);
  glEnable(GL_LIGHTING);

  glDisable(GL_LIGHT0);
  glEnable(GL_LIGHT1);
  glutSolidSphere(0.55,12,12);
  glDisable(GL_LIGHT1);
  glEnable(GL_LIGHT0);
		
  glColor4f(1.0,0.1,0.2,0.1);
  glutSolidSphere(0.55+.25*fabs(sin(yRot/25.)),30,30);
		
  glDisable(GL_LIGHTING);
  glColor4f(1.0,1.0,0.0,0.01);
  glutSolidSphere(0.55+2.*fabs(sin(yRot/25.)),30,30);
  glEnable(GL_LIGHTING);
		
  glPopMatrix();

  glutSwapBuffers();

  yRot += 0.5;
}

static void initFaisceaux() {
  int i;
  time_t t;
  time(&t);
  srand(t);
  for(i = 0; i < NB_FAISCEAUX; i++) {
    ds[i] = i*PI/360.;
    dt[i] = (rand()%360)*PI/180.;
  }
}

static void idle(void) {
  glutPostRedisplay();
}

static void reshape(GLsizei width, GLsizei height) {
  GLfloat h = (GLfloat) height / (GLfloat) width;

  glViewport(0, 0,  width,  height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum(-1.0, 1.0, -h, h, 2.0, 10.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

void initLights() {
  glLightfv(GL_LIGHT0, GL_AMBIENT, ambientColor);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseColor);
  glLightfv(GL_LIGHT0, GL_POSITION, lumierePosition);
  glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.05);

  glEnable(GL_LIGHTING);//active l'ombrage
  glEnable(GL_LIGHT0);//active la lumire 0


  glLightfv(GL_LIGHT1, GL_AMBIENT, redColor);
  glLightfv(GL_LIGHT1, GL_POSITION, lumierePosition2);
  glDisable(GL_LIGHT1);
}

static void init(void) {
  glEnable(GL_DEPTH_TEST);
  glDisable(GL_CULL_FACE);

  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	
  glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
  glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);
  glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

  initLights();
  initFaisceaux();
}

void visible(int vis) {
  if (vis == GLUT_VISIBLE)
    glutIdleFunc(idle);
  else
    glutIdleFunc(NULL);
}

void key(unsigned char k, int x, int y) {
  if(k == 27) {
    exit(0);
  }
}

int main(int argc, char *argv[]) {
  glutInit(&argc, argv);

  glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
  glutInitWindowSize(320,200);
  glutCreateWindow("Demo GL");
  init();

  glutDisplayFunc(render);
  glutReshapeFunc(reshape);
  glutKeyboardFunc(key);
  glutVisibilityFunc(visible);

  glutMainLoop();
  return 0;
}
